@@ -1,8 +1,8 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "bufferfile" "1.0.6"
+(define-package "bufferfile" "20251011.1158"
   "Rename/Delete/Copy Files and Associated Buffers."
   '((emacs "26.1"))
   :url "https://github.com/jamescherti/bufferfile.el"
-  :commit "5a78bf05443d60d651aaadcb8855d84b39bab182"
-  :revdesc "1.0.6-0-g5a78bf05443d"
+  :commit "e4e5c46fcf656fa06f7388b514fbf17636bba424"
+  :revdesc "e4e5c46fcf65"
   :keywords '("convenience"))
