@@ -3,8 +3,8 @@
 ;; Copyright (C) 2024-2025 James Cherti | https://www.jamescherti.com/contact/
 
 ;; Author: James Cherti
-;; Package-Version: 1.0.6
-;; Package-Revision: 1.0.6-0-g5a78bf05443d
+;; Package-Version: 20251011.1158
+;; Package-Revision: e4e5c46fcf65
 ;; URL: https://github.com/jamescherti/bufferfile.el
 ;; Keywords: convenience
 ;; Package-Requires: ((emacs "26.1"))
@@ -91,20 +91,24 @@ Possible values are:
                  (const :tag "Do nothing" nil))
   :group 'bufferfile)
 
-(defcustom bufferfile-dired-integration t
-  "Whether to enable Dired integration.
-Dired must be loaded for this option to have any effect."
-  :type 'boolean
-  :group 'bufferfile)
-
-(defcustom bufferfile-eglot-integration t
-  "Whether to enable Eglot integration.
-Eglot must be loaded for this option to have any effect."
+(defcustom bufferfile-make-target-directory t
+  "If non-nil, ensure the destination directory exists.
+This applies to file operations such as renaming or copying."
   :type 'boolean
   :group 'bufferfile)
 
 ;;; Variables
 
+(defvar bufferfile-dired-integration t
+  "Whether to enable Dired integration.
+This refreshes all Dired buffers containing the affected files.
+Dired must be loaded for this option to have any effect.")
+
+(defvar bufferfile-eglot-integration t
+  "Whether to enable Eglot integration.
+This reloads Eglot to ensure it references the updated file name.
+Eglot must be loaded for this option to have any effect.")
+
 (defvar bufferfile-pre-rename-functions nil
   "Hook run before renaming a file.
 Each function receives three arguments: (previous-path new-path list-buffers).")
@@ -125,14 +129,16 @@ Each function receives two arguments: (path list_buffers).")
   "Prefix used for messages and errors related to bufferfile operations.")
 
 ;; Internal
+
 (defvar-local bufferfile--dired-file-selected nil)
 
 ;;; Helper functions
 
-(defun bufferfile--error (&rest args)
+(defmacro bufferfile--error (&rest args)
   "Signal an error with `bufferfile-message-prefix' followed by formatted ARGS.
 ARGS are formatted as in `format'."
-  (user-error "%s%s" bufferfile-message-prefix (apply #'format args)))
+  (declare (indent 0) (debug t))
+  `(user-error "%s%s" bufferfile-message-prefix (format ,@args)))
 
 (defun bufferfile--message (&rest args)
   "Display a message with '[bufferfile]' prepended.
@@ -200,7 +206,7 @@ PROMPT-PREFIX: The text prepended to the user input prompt."
     (when (string= (file-truename filename)
                    (file-truename new-filename))
       (bufferfile--error
-       "Ignored because the destination is the same as the source"))
+        "Ignored because the destination is the same as the source"))
     new-filename))
 
 (defun bufferfile--read-dest-file-name-rename (filename ok-if-already-exists)
@@ -215,8 +221,8 @@ is non-nil."
                 "Destination file '%s' already exists. Do you want to overwrite it?"
                 new-filename))
         (bufferfile--error
-         "Rename failed: Destination filename already exists: %s"
-         new-filename)))
+          "Rename failed: Destination filename already exists: %s"
+          new-filename)))
 
     new-filename))
 
@@ -288,15 +294,7 @@ buffer."
           (when bufferfile-verbose
             (bufferfile--message "dired-revert: %s" default-directory))
           (ignore-errors
-            (funcall 'dired-revert)))
-
-        ;; (when (and goto-file
-        ;;            (fboundp 'dired-goto-file)
-        ;;            bufferfile--dired-file-selected)
-        ;;   (when bufferfile-verbose
-        ;;     (bufferfile--message "dired-goto-file: %s" goto-file))
-        ;;   (funcall 'dired-goto-file goto-file))
-        )))
+            (funcall 'dired-revert))))))
 
   ;; Walk windows
   (walk-windows
@@ -304,7 +302,6 @@ buffer."
      (when window
        (with-selected-window window
          (when (derived-mode-p 'dired-mode)
-           (bufferfile--message "TESTTESTdired-goto-file: %s" goto-file)
            (when (and goto-file
                       (fboundp 'dired-goto-file)
                       bufferfile--dired-file-selected)
@@ -318,6 +315,15 @@ buffer."
 
 ;;; Rename file
 
+(defun bufferfile--generate-buffer-name (&optional newname)
+  "Return a unique buffer name based on NEWNAME or base buffer name.
+Normalizes any trailing <n> suffix. Does not rename the current buffer."
+  (setq newname (or newname (buffer-name (buffer-base-buffer))))
+  (if (string-match "<[0-9]+>\\'" newname)
+      (setq newname (substring newname 0 (match-beginning 0))))
+  (setq newname (generate-new-buffer-name newname))
+  newname)
+
 (defun bufferfile--rename-all-buffers (old-filename new-filename)
   "Update buffer names and files they are visiting to reflect the renaming.
 OLD-FILENAME and NEW-FILENAME are absolute paths as returned by
@@ -330,47 +336,41 @@ This includes indirect buffers whose names are derived from the old filename."
   (setq old-filename (expand-file-name old-filename))
   (setq new-filename (expand-file-name new-filename))
 
-  (let ((indirect-buffers '()))
-    ;; Update the file name associated with buffers visiting files
+  ;; Update the file name associated with buffers visiting files
+  (let (file-visiting-buffers
+        indirect-buffers
+        (old-filename-truename (file-truename old-filename)))
     (dolist (buf (buffer-list))
       (when (buffer-live-p buf)
         (with-current-buffer buf
           (let ((base-buffer (buffer-base-buffer)))
             (if base-buffer
-                ;; Indirect buffer (clone)
-                (when (buffer-file-name base-buffer)
-                  (push buf indirect-buffers))
-              ;; File-visiting buffer
+                (push buf indirect-buffers)
+              (push buf file-visiting-buffers))
+
+            ;; File-visiting buffer
+            (when (not base-buffer)
               (when (and buffer-file-name
                          (string= (file-truename buffer-file-name)
-                                  (file-truename old-filename)))
+                                  old-filename-truename))
                 (set-visited-file-name new-filename t t)))))))
 
-    ;; Update the names of indirect (clone) buffers associated with buffers
-    ;; visiting the renamed files.
-    (when indirect-buffers
-      (let ((basename (file-name-nondirectory old-filename))
-            (new-basename (file-name-nondirectory new-filename)))
-        (dolist (buf indirect-buffers)
-          (with-current-buffer buf
-            (when-let* ((base-buffer (buffer-base-buffer)))
-              (let ((base-buffer-filename
-                     (let ((file-name (buffer-file-name base-buffer)))
-                       (when file-name
-                         (expand-file-name file-name))))
-                    (indirect-buffer-name (buffer-name)))
-                (when (and base-buffer-filename
-                           (string= (file-truename base-buffer-filename)
-                                    (file-truename new-filename)))
-                  (when (string-prefix-p basename indirect-buffer-name)
-                    (rename-buffer (concat new-basename
-                                           (substring indirect-buffer-name
-                                                      (length basename)))
-                                   t)))))))))))
+    ;; Update the names of file visiting buffer and indirect buffers (clones)
+    ;; associated with buffers visiting the renamed files.
+    (let ((new-filename-truename (file-truename new-filename)))
+      (dolist (buf indirect-buffers)
+        (with-current-buffer buf
+          (when-let* ((base-buffer (buffer-base-buffer)))
+            (let ((base-buffer-filename (buffer-file-name base-buffer)))
+              (when (and base-buffer-filename
+                         (string= (file-truename base-buffer-filename)
+                                  new-filename-truename))
+                (rename-buffer (bufferfile--generate-buffer-name) t)))))))))
 
 (defun bufferfile-rename-file (filename
                                new-filename
-                               &optional ok-if-already-exists)
+                               &optional ok-if-already-exists
+                               confirm-overwrite)
   "Rename FILENAME to NEW-FILENAME.
 
 This function updates:
@@ -382,13 +382,32 @@ Hooks in `bufferfile-pre-rename-functions' and
 `bufferfile-post-rename-functions' are run before and after the renaming
 process.
 
-Signal an error if a filename NEWNAME already exists unless OK-IF-ALREADY-EXISTS
-is non-nil."
-  (let (list-buffers)
-    (when (and (not ok-if-already-exists)
-               (file-exists-p new-filename))
-      (error "%sDestination file '%s' already exists"
-             bufferfile-message-prefix new-filename))
+When CONFIRM-OVERWRITE is non-nil, prompt the user for confirmation before
+overwriting existing files.
+
+Signal an error if NEW-FILENAME already exists unless OK-IF-ALREADY-EXISTS is
+non-nil."
+  (let ((src-truename (file-truename filename))
+        (dst-truename (file-truename new-filename))
+        list-buffers)
+    (when (string= src-truename dst-truename)
+      (bufferfile--error "Source and destination are the same file: '%s'"
+                         src-truename))
+
+    (unless (file-exists-p filename)
+      (bufferfile--error "Source file '%s' does not exist; cannot move to '%s'"
+                         filename new-filename))
+
+    (when (file-exists-p new-filename)
+      (if (and (not ok-if-already-exists)
+               confirm-overwrite)
+          (if (y-or-n-p
+               (format "Destination file '%s' already exists. Overwrite? "
+                       new-filename))
+              (setq ok-if-already-exists t)
+            (bufferfile--error "Rename aborted"))
+        (bufferfile--error "Destination file '%s' already exists"
+                           new-filename)))
 
     (setq list-buffers (bufferfile--get-list-buffers filename))
 
@@ -412,6 +431,11 @@ is non-nil."
                             (and file (string= (file-truename file)
                                                filename-truename)))))))))
 
+    ;; Ensure that the destination directory exists
+    (when bufferfile-make-target-directory
+      (when-let* ((dest-dir (file-name-directory new-filename)))
+        (make-directory dest-dir t)))
+
     (if (and bufferfile-use-vc
              (vc-backend filename)
              (let ((root1
@@ -503,7 +527,35 @@ process."
         (let ((new-filename
                (bufferfile--read-dest-file-name-rename filename
                                                        ok-if-already-exists)))
-          (bufferfile-rename-file filename new-filename t))))))
+          (bufferfile-rename-file filename new-filename nil t))))))
+
+;;; Dired do rename
+
+(defun bufferfile-dired-do-rename (&optional arg)
+  "Rename the current file or all marked files in Dired.
+ARG is the same argument as `dired-do-rename'.
+If one file is marked, delegate to `bufferfile-rename'.
+If multiple files are marked, delegate to `dired-do-rename'."
+  (interactive "P")
+  (when (and (derived-mode-p 'dired-mode)
+             (fboundp 'dired-get-marked-files)
+             (fboundp 'dired-post-do-command))
+    (let* ((marked-files (dired-get-marked-files nil arg))
+           new-filename
+           old-filename)
+      (if (> (length marked-files) 1)
+          (when (fboundp 'dired-do-rename)
+            (dired-do-rename arg))
+        (progn
+          (when (or (not marked-files) (= (length marked-files) 0))
+            (bufferfile--error "You need to select at least one file"))
+
+          (setq old-filename (car marked-files))
+          (setq new-filename
+                (bufferfile--read-dest-file-name-rename old-filename t))
+
+          (when new-filename
+            (bufferfile-rename-file old-filename new-filename t)))))))
 
 ;;; Delete file
 
@@ -542,7 +594,8 @@ process."
                (list-buffers (bufferfile--get-list-buffers filename))
                (parent-dir-path (file-name-directory filename)))
           (unless parent-dir-path
-            (error "Cannot find the parent directory of: %s" filename))
+            (bufferfile--error "Cannot find the parent directory of: %s"
+                               filename))
           (dolist (buf list-buffers)
             (with-current-buffer buf
               (when (buffer-modified-p)
@@ -600,7 +653,7 @@ process."
                 ;; VC delete
                 (bufferfile--vc-delete-file filename)
               ;; Delete
-              (delete-file filename)))
+              (delete-file filename delete-by-moving-to-trash)))
 
           (when bufferfile-verbose
             (bufferfile--message "Deleted: %s" (abbreviate-file-name filename)))
@@ -624,46 +677,32 @@ process."
   (with-current-buffer buffer
     (let* ((filename (bufferfile--get-buffer-filename))
            (original-buffer (or (buffer-base-buffer) (current-buffer))))
+      ;; Save
       (with-current-buffer original-buffer
         (when (buffer-modified-p)
           (let ((save-silently (not bufferfile-verbose)))
             (save-buffer)))
 
+        ;; Prompt user
         (let ((new-filename (bufferfile--read-dest-file-name filename
                                                              "Copy ")))
           (when bufferfile-verbose
             (bufferfile--message "Copy: %s -> %s"
                                  (abbreviate-file-name filename)
                                  (abbreviate-file-name new-filename)))
-          (copy-file filename new-filename t))))))
 
-;;; Dired do rename
+          ;; Ensure that the destination directory exists
+          (when bufferfile-make-target-directory
+            (when-let* ((dest-dir (file-name-directory new-filename)))
+              (make-directory dest-dir t)))
 
-(defun bufferfile-dired-do-rename (&optional arg)
-  "Rename the current file or all marked files in Dired.
-ARG is the same argument as `dired-do-rename'.
-If one file is marked, delegate to `bufferfile-rename'.
-If multiple files are marked, delegate to `dired-do-rename'."
-  (interactive "P")
-  (when (and (derived-mode-p 'dired-mode)
-             (fboundp 'dired-get-marked-files)
-             (fboundp 'dired-post-do-command))
-    (let* ((marked-files (dired-get-marked-files nil arg))
-           new-filename
-           old-filename)
-      (if (> (length marked-files) 1)
-          (when (fboundp 'dired-do-rename)
-            (dired-do-rename arg))
-        (progn
-          (when (or (not marked-files) (= (length marked-files) 0))
-            (user-error "You need to select at least one file"))
+          (copy-file filename new-filename t)
 
-          (setq old-filename (car marked-files))
-          (setq new-filename
-                (bufferfile--read-dest-file-name-rename old-filename t))
-
-          (when new-filename
-            (bufferfile-rename-file old-filename new-filename t)))))))
+          ;; Refresh dired buffers
+          (when bufferfile-dired-integration
+            ;; Refresh the dired buffer
+            (let ((parent-dir-path (file-name-directory (expand-file-name new-filename))))
+              (bufferfile--refresh-dired-buffers parent-dir-path new-filename))))))))
 
 ;;; Provide
 (provide 'bufferfile)
